package main.java.com.djrapitops.plan.data;

/**
 * This class is used for storing start and end of a playsession inside UserData
 * object.
 *
 * @author Rsl1122
 */
public class SessionData {

    private long sessionStart;
    private long sessionEnd;

    /**
     * Creates a new session with given start and end of -1.
     *
     * @param sessionStart Epoch millisecond the session was started.
     */
    public SessionData(long sessionStart) {
        this.sessionStart = sessionStart;
        this.sessionEnd = -1;
    }

    /**
     * Creates a new session with given start and end.
     *
     * @param sessionStart Epoch millisecond the session was started.
     * @param sessionEnd Epoch millisecond the session ended.
     */
    public SessionData(long sessionStart, long sessionEnd) {
        this.sessionStart = sessionStart;
        this.sessionEnd = sessionEnd;
    }

    /**
     * Ends the session with given end point.
     *
     * (Changes the end to the parameter.).
     *
     * @param endOfSession Epoch millisecond the session ended.
     */
    public void endSession(long endOfSession) {
        sessionEnd = endOfSession;
    }

    /**
     * Get the start of the session.
     *
     * @return Epoch millisecond the session started.
     */
    public long getSessionStart() {
        return sessionStart;
    }

    /**
     * Get the end of the session.
     *
     * @return Epoch millisecond the session ended.
     */
    public long getSessionEnd() {
        return sessionEnd;
    }

    /**
     * Get the length of the session in milliseconds.
     *
     * @return Long in ms.
     */
    public long getLength() {
        return sessionEnd - sessionStart;
    }

    @Override
    public String toString() {
        return "s:" + sessionStart + " e:" + sessionEnd;
    }

    /**
     * Check if the session start was before the end.
     *
     * @return Is the length positive?
     */
    public boolean isValid() {
        return sessionStart <= sessionEnd;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final SessionData other = (SessionData) obj;
        if (this.sessionStart != other.sessionStart) {
            return false;
        }
        if (this.sessionEnd != other.sessionEnd) {
            return false;
        }
        return true;
    }
}
